@@ -2,10 +2,11 @@ use std::env;
 use std::error::Error;
 use std::fmt;
 use std::fs;
-use std::io::{self, prelude::*, Write};
+use std::io::{self, Write};
 use std::path::{Path, PathBuf};
 use std::time::Duration;
 
+use chrono::{SecondsFormat, Utc};
 use encoding_rs::Encoding;
 use markup5ever_rcdom::RcDom;
 use reqwest::blocking::Client;
@@ -15,8 +16,9 @@ use url::Url;
 use crate::cache::Cache;
 use crate::cookies::Cookie;
 use crate::html::{
-    add_favicon, create_metadata_tag, get_base_url, get_charset, has_favicon, html_to_dom,
-    serialize_document, set_base_url, set_charset, walk_and_embed_assets,
+    add_favicon, create_metadata_tag, get_base_url, get_charset, get_robots, get_title,
+    has_favicon, html_to_dom, serialize_document, set_base_url, set_charset, set_robots,
+    walk_and_embed_assets,
 };
 use crate::url::{clean_url, create_data_url, get_referer_url, parse_data_url, resolve_url};
 
@@ -59,7 +61,7 @@ pub enum MonolithOutputFormat {
 pub struct Options {
     pub base_url: Option<String>,
     pub blacklist_domains: bool,
-    pub cookies: Vec<Cookie>, // TODO: move out of this struct
+    pub cookies: Vec<Cookie>, // TODO: move out of this struct?
     pub domains: Option<Vec<String>>,
     pub encoding: Option<String>,
     pub ignore_errors: bool,
@@ -105,86 +107,19 @@ const FILE_SIGNATURES: [[&[u8]; 2]; 18] = [
     [b"....moov", b"video/quicktime"],
     [b"\x1A\x45\xDF\xA3", b"video/webm"],
 ];
+// All known non-"text/..." plaintext media types
 const PLAINTEXT_MEDIA_TYPES: &[&str] = &[
-    "application/javascript",
-    "application/json",
-    "image/svg+xml",
+    "application/javascript",          // .js
+    "application/json",                // .json
+    "application/ld+json",             // .jsonld
+    "application/x-sh",                // .sh
+    "application/xhtml+xml",           // .xhtml
+    "application/xml",                 // .xml
+    "application/vnd.mozilla.xul+xml", // .xul
+    "image/svg+xml",                   // .svg
 ];
 
-pub fn create_monolithic_document(
-    source: String,
-    options: &Options,
-    cache: &mut Option<Cache>,
-) -> Result<Vec<u8>, MonolithError> {
-    // Check if source was provided
-    if source.is_empty() {
-        return Err(MonolithError::new("no target specified"));
-    }
-
-    // Check if custom encoding value is acceptable
-    if let Some(custom_encoding) = options.encoding.clone() {
-        if Encoding::for_label_no_replacement(custom_encoding.as_bytes()).is_none() {
-            return Err(MonolithError::new(&format!(
-                "unknown encoding \"{}\"",
-                &custom_encoding
-            )));
-        }
-    }
-
-    let mut use_stdin: bool = false;
-
-    let target_url = match source.as_str() {
-        "-" => {
-            // Read from pipe (stdin)
-            use_stdin = true;
-            // Set default target URL to an empty data URL; the user can set it via --base-url
-            Url::parse("data:text/html,").unwrap()
-        }
-        target => match Url::parse(target) {
-            Ok(url) => match url.scheme() {
-                "data" | "file" | "http" | "https" => url,
-                unsupported_scheme => {
-                    return Err(MonolithError::new(&format!(
-                        "unsupported target URL scheme \"{}\"",
-                        unsupported_scheme
-                    )));
-                }
-            },
-            Err(_) => {
-                // Failed to parse given base URL (perhaps it's a filesystem path?)
-                let path: &Path = Path::new(&target);
-                match path.exists() {
-                    true => match path.is_file() {
-                        true => {
-                            let canonical_path = fs::canonicalize(path).unwrap();
-                            match Url::from_file_path(canonical_path) {
-                                Ok(url) => url,
-                                Err(_) => {
-                                    return Err(MonolithError::new(&format!(
-                                        "could not generate file URL out of given path \"{}\"",
-                                        &target
-                                    )));
-                                }
-                            }
-                        }
-                        false => {
-                            return Err(MonolithError::new(&format!(
-                                "local target \"{}\" is not a file",
-                                &target
-                            )));
-                        }
-                    },
-                    false => {
-                        // It is not a FS path, now we do what browsers do:
-                        // prepend "http://" and hope it points to a website
-                        Url::parse(&format!("http://{}", &target)).unwrap()
-                    }
-                }
-            }
-        },
-    };
-
-    // Initialize HTTP client
+pub fn init_client(options: &Options) -> Client {
     let mut header_map = HeaderMap::new();
     if let Some(user_agent) = &options.user_agent {
         header_map.insert(
@@ -192,7 +127,7 @@ pub fn create_monolithic_document(
             HeaderValue::from_str(user_agent).expect("Invalid User-Agent header specified"),
         );
     }
-    let client = Client::builder()
+    Client::builder()
         .timeout(Duration::from_secs(if options.timeout > 0 {
             options.timeout
         } else {
@@ -203,85 +138,68 @@ pub fn create_monolithic_document(
         .danger_accept_invalid_certs(options.insecure)
         .default_headers(header_map)
         .build()
-        .expect("Failed to initialize HTTP client");
+        .expect("Failed to initialize HTTP client")
+}
 
-    // At first we assume that base URL is same as target URL
-    let mut base_url: Url = target_url.clone();
-
-    let data: Vec<u8>;
-    let mut document_encoding: String = "".to_string();
-    let mut dom: RcDom;
-
-    // Retrieve target document
-    if use_stdin {
-        data = read_stdin();
-    } else if target_url.scheme() == "file"
-        || target_url.scheme() == "http"
-        || target_url.scheme() == "https"
-        || target_url.scheme() == "data"
+pub fn create_monolithic_document_from_data(
+    input_data: Vec<u8>,
+    options: &Options,
+    cache: &mut Option<Cache>,
+    input_encoding: Option<String>,
+    input_target: Option<String>,
+) -> Result<(Vec<u8>, Option<String>), MonolithError> {
+    // Validate options
     {
-        match retrieve_asset(cache, &client, &target_url, &target_url, options) {
-            Ok((retrieved_data, final_url, media_type, charset)) => {
-                // Provide output as text (without processing it, the way browsers do)
-                if !media_type.eq_ignore_ascii_case("text/html")
-                    && !media_type.eq_ignore_ascii_case("application/xhtml+xml")
-                {
-                    return Ok(retrieved_data);
-                }
-
-                if options
-                    .base_url
-                    .clone()
-                    .unwrap_or("".to_string())
-                    .is_empty()
-                {
-                    base_url = final_url;
-                }
-
-                data = retrieved_data;
-                document_encoding = charset;
-            }
-            Err(_) => {
-                return Err(MonolithError::new("could not retrieve target document"));
+        // Check if custom encoding value is acceptable
+        if let Some(custom_output_encoding) = options.encoding.clone() {
+            if Encoding::for_label_no_replacement(custom_output_encoding.as_bytes()).is_none() {
+                return Err(MonolithError::new(&format!(
+                    "unknown encoding \"{}\"",
+                    &custom_output_encoding
+                )));
             }
         }
-    } else {
-        return Err(MonolithError::new("unsupported target"));
     }
 
+    let client: Client = init_client(options);
+    let mut base_url: Url = if input_target.is_some() {
+        Url::parse(&input_target.clone().unwrap()).unwrap()
+    } else {
+        Url::parse("data:text/html,").unwrap()
+    };
+    let mut document_encoding: String = input_encoding.clone().unwrap_or("utf-8".to_string());
+    let mut dom: RcDom;
+
     // Initial parse
-    dom = html_to_dom(&data, document_encoding.clone());
+    dom = html_to_dom(&input_data, document_encoding.clone());
 
-    // TODO: investigate if charset from filesystem/data URL/HTTP headers
-    //       has say over what's specified in HTML
-
-    // Attempt to determine document's charset
+    // Attempt to determine document's encoding
     if let Some(html_charset) = get_charset(&dom.document) {
         if !html_charset.is_empty() {
             // Check if the charset specified inside HTML is valid
-            if let Some(encoding) = Encoding::for_label_no_replacement(html_charset.as_bytes()) {
+            if let Some(document_charset) =
+                Encoding::for_label_no_replacement(html_charset.as_bytes())
+            {
                 document_encoding = html_charset;
-                dom = html_to_dom(&data, encoding.name().to_string());
+                dom = html_to_dom(&input_data, document_charset.name().to_string());
             }
         }
     }
 
-    // Use custom base URL if specified, read and use what's in the DOM otherwise
-    let custom_base_url: String = options.base_url.clone().unwrap_or("".to_string());
+    // Use custom base URL if specified; read and use what's in the DOM otherwise
+    let custom_base_url: String = options.base_url.clone().unwrap_or_default();
     if custom_base_url.is_empty() {
-        // No custom base URL is specified
-        // Try to see if document has BASE element
+        // No custom base URL is specified; try to see if document has BASE element
         if let Some(existing_base_url) = get_base_url(&dom.document) {
-            base_url = resolve_url(&target_url, &existing_base_url);
+            base_url = resolve_url(&base_url, &existing_base_url);
         }
     } else {
         // Custom base URL provided
         match Url::parse(&custom_base_url) {
             Ok(parsed_url) => {
                 if parsed_url.scheme() == "file" {
-                    // File base URLs can only work with
-                    // documents saved from filesystem
-                    if target_url.scheme() == "file" {
+                    // File base URLs can only work with documents saved from filesystem
+                    if base_url.scheme() == "file" {
                         base_url = parsed_url;
                     }
                 } else {
@@ -290,7 +208,7 @@ pub fn create_monolithic_document(
             }
             Err(_) => {
                 // Failed to parse given base URL, perhaps it's a filesystem path?
-                if target_url.scheme() == "file" {
+                if base_url.scheme() == "file" {
                     // Relative paths could work for documents saved from filesystem
                     let path: &Path = Path::new(&custom_base_url);
                     if path.exists() {
@@ -321,12 +239,21 @@ pub fn create_monolithic_document(
 
     // Request and embed /favicon.ico (unless it's already linked in the document)
     if !options.no_images
-        && (target_url.scheme() == "http" || target_url.scheme() == "https")
+        && (base_url.scheme() == "http" || base_url.scheme() == "https")
+        && (input_target.is_some()
+            && (input_target.as_ref().unwrap().starts_with("http:")
+                || input_target.as_ref().unwrap().starts_with("https:")))
         && !has_favicon(&dom.document)
     {
         let favicon_ico_url: Url = resolve_url(&base_url, "/favicon.ico");
 
-        match retrieve_asset(cache, &client, &target_url, &favicon_ico_url, options) {
+        match retrieve_asset(
+            cache,
+            &client,
+            /*&target_url, */ &base_url,
+            &favicon_ico_url,
+            options,
+        ) {
             Ok((data, final_url, media_type, charset)) => {
                 let favicon_data_url: Url =
                     create_data_url(&media_type, &charset, &data, &final_url);
@@ -338,29 +265,154 @@ pub fn create_monolithic_document(
         }
     }
 
+    // Append noindex META-tag
+    if let meta_robots_content_value = get_robots(&dom.document).unwrap_or_default() {
+        if meta_robots_content_value.trim().is_empty() || meta_robots_content_value != "none" {
+            dom = set_robots(dom, "none");
+        }
+    }
+
     // Save using specified charset, if given
     if let Some(custom_encoding) = options.encoding.clone() {
         document_encoding = custom_encoding;
         dom = set_charset(dom, document_encoding.clone());
     }
 
+    let document_title: Option<String> = get_title(&dom.document);
+
     if options.output_format == MonolithOutputFormat::HTML {
         // Serialize DOM tree
         let mut result: Vec<u8> = serialize_document(dom, document_encoding, options);
 
         // Prepend metadata comment tag
-        if !options.no_metadata {
-            let mut metadata_comment: String = create_metadata_tag(&target_url);
+        if !options.no_metadata && !input_target.clone().unwrap_or_default().is_empty() {
+            let mut metadata_comment: String =
+                create_metadata_tag(&Url::parse(&input_target.unwrap_or_default()).unwrap());
+            // let mut metadata_comment: String = create_metadata_tag(target);
             metadata_comment += "\n";
             result.splice(0..0, metadata_comment.as_bytes().to_vec());
         }
 
-        Ok(result)
+        Ok((result, document_title))
     } else {
-        Ok(vec![])
+        Ok((vec![], document_title))
     }
 }
 
+pub fn create_monolithic_document(
+    target: String,
+    options: &mut Options,
+    cache: &mut Option<Cache>,
+) -> Result<(Vec<u8>, Option<String>), MonolithError> {
+    // Check if target was provided
+    if target.is_empty() {
+        return Err(MonolithError::new("no target specified"));
+    }
+
+    // Validate options
+    {
+        // Check if custom encoding value is acceptable
+        if let Some(custom_encoding) = options.encoding.clone() {
+            if Encoding::for_label_no_replacement(custom_encoding.as_bytes()).is_none() {
+                return Err(MonolithError::new(&format!(
+                    "unknown encoding \"{}\"",
+                    &custom_encoding
+                )));
+            }
+        }
+    }
+
+    let mut target_url = match target.as_str() {
+        target_str => match Url::parse(target_str) {
+            Ok(target_url) => match target_url.scheme() {
+                "data" | "file" | "http" | "https" => target_url,
+                unsupported_scheme => {
+                    return Err(MonolithError::new(&format!(
+                        "unsupported target URL scheme \"{}\"",
+                        unsupported_scheme
+                    )));
+                }
+            },
+            Err(_) => {
+                // Failed to parse given base URL (perhaps it's a filesystem path?)
+                let path: &Path = Path::new(&target_str);
+
+                match path.exists() {
+                    true => match path.is_file() {
+                        true => {
+                            let canonical_path = fs::canonicalize(path).unwrap();
+
+                            match Url::from_file_path(canonical_path) {
+                                Ok(url) => url,
+                                Err(_) => {
+                                    return Err(MonolithError::new(&format!(
+                                        "could not generate file URL out of given path \"{}\"",
+                                        &target_str
+                                    )));
+                                }
+                            }
+                        }
+                        false => {
+                            return Err(MonolithError::new(&format!(
+                                "local target \"{}\" is not a file",
+                                &target_str
+                            )));
+                        }
+                    },
+                    false => {
+                        // It is not a FS path, now we do what browsers do:
+                        // prepend "http://" and hope it points to a website
+                        Url::parse(&format!("http://{}", &target_str)).unwrap()
+                    }
+                }
+            }
+        },
+    };
+
+    let client: Client = init_client(options);
+    let data: Vec<u8>;
+    let document_encoding: Option<String>;
+
+    // Retrieve target document
+    if target_url.scheme() == "file"
+        || target_url.scheme() == "http"
+        || target_url.scheme() == "https"
+        || target_url.scheme() == "data"
+    {
+        match retrieve_asset(cache, &client, &target_url, &target_url, options) {
+            Ok((retrieved_data, final_url, media_type, charset)) => {
+                if !media_type.eq_ignore_ascii_case("text/html")
+                    && !media_type.eq_ignore_ascii_case("application/xhtml+xml")
+                {
+                    // Provide output as text (without processing it, the way browsers do)
+                    return Ok((retrieved_data, None));
+                }
+
+                // If got redirected, set target_url to that
+                if final_url != target_url {
+                    target_url = final_url.clone();
+                }
+
+                data = retrieved_data;
+                document_encoding = Some(charset);
+            }
+            Err(_) => {
+                return Err(MonolithError::new("could not retrieve target document"));
+            }
+        }
+    } else {
+        return Err(MonolithError::new("unsupported target"));
+    }
+
+    create_monolithic_document_from_data(
+        data,
+        options,
+        cache,
+        document_encoding,
+        Some(target_url.to_string()),
+    )
+}
+
 pub fn detect_media_type(data: &[u8], url: &Url) -> String {
     // At first attempt to read file's header
     for file_signature in FILE_SIGNATURES.iter() {
@@ -389,8 +441,9 @@ pub fn detect_media_type_by_file_name(filename: &str) -> String {
             "htm" | "html" => "text/html",
             "ico" => "image/x-icon",
             "jpeg" | "jpg" => "image/jpeg",
-            "js" => "application/javascript",
+            "js" => "text/javascript",
             "json" => "application/json",
+            "jsonld" => "application/ld+json",
             "mp3" => "audio/mpeg",
             "mp4" | "m4v" => "video/mp4",
             "ogg" => "audio/ogg",
@@ -405,12 +458,12 @@ pub fn detect_media_type_by_file_name(filename: &str) -> String {
             "webp" => "image/webp",
             "woff" => "font/woff",
             "woff2" => "font/woff2",
+            "xhtml" => "application/xhtml+xml",
             "xml" => "text/xml",
             &_ => "",
         },
         None => "",
     };
-
     mime.to_string()
 }
 
@@ -470,6 +523,34 @@ pub fn domain_is_within_domain(domain: &str, domain_to_match_against: &str) -> b
     ok
 }
 
+pub fn format_output_path(destination: &str, document_title: &str) -> String {
+    let datetime: &str = &Utc::now().to_rfc3339_opts(SecondsFormat::Secs, true);
+
+    destination
+        .replace("%timestamp%", &datetime.replace(':', "_"))
+        .replace(
+            "%title%",
+            document_title
+                .to_string()
+                .replace(['/', '\\'], "_")
+                .replace('<', "[")
+                .replace('>', "]")
+                .replace(':', " - ")
+                .replace('\"', "")
+                .replace('|', "-")
+                .replace('?', "")
+                .trim_start_matches('.'),
+        )
+        .to_string()
+        .replace('<', "[")
+        .replace('>', "]")
+        .replace(':', " - ")
+        .replace('\"', "")
+        .replace('|', "-")
+        .replace('?', "")
+        .to_string()
+}
+
 pub fn is_plaintext_media_type(media_type: &str) -> bool {
     media_type.to_lowercase().as_str().starts_with("text/")
         || PLAINTEXT_MEDIA_TYPES.contains(&media_type.to_lowercase().as_str())
@@ -511,9 +592,11 @@ pub fn retrieve_asset(
         let (media_type, charset, data) = parse_data_url(url);
         Ok((data, url.clone(), media_type, charset))
     } else if url.scheme() == "file" {
+        let cache_key: String = clean_url(url.clone()).as_str().to_string();
+
         // Check if parent_url is also a file: URL (if not, then we don't embed the asset)
         if parent_url.scheme() != "file" {
-            print_error_message(&format!("{} (security error)", &url), options);
+            print_error_message(&format!("{} (security error)", &cache_key), options);
 
             // Provoke error
             client.get("").send()?;
@@ -523,12 +606,12 @@ pub fn retrieve_asset(
         let path: &Path = path_buf.as_path();
         if path.exists() {
             if path.is_dir() {
-                print_error_message(&format!("{} (is a directory)", &url), options);
+                print_error_message(&format!("{} (is a directory)", &cache_key), options);
 
                 // Provoke error
                 Err(client.get("").send().unwrap_err())
             } else {
-                print_info_message(&format!("{}", &url), options);
+                print_info_message(&cache_key.to_string(), options);
 
                 let file_blob: Vec<u8> = fs::read(path).expect("unable to read file");
 
@@ -656,15 +739,6 @@ pub fn retrieve_asset(
     }
 }
 
-pub fn read_stdin() -> Vec<u8> {
-    let mut buffer: Vec<u8> = vec![];
-
-    match io::stdin().lock().read_to_end(&mut buffer) {
-        Ok(_) => buffer,
-        Err(_) => buffer,
-    }
-}
-
 pub fn print_error_message(text: &str, options: &Options) {
     if !options.silent {
         let stderr = io::stderr();
