@@ -1,7 +1,7 @@
+pub mod cache;
 pub mod cookies;
+pub mod core;
 pub mod css;
 pub mod html;
 pub mod js;
-pub mod opts;
 pub mod url;
-pub mod utils;
