@@ -10,16 +10,16 @@ mod passing {
     use html5ever::serialize::{serialize, SerializeOpts};
     use markup5ever_rcdom::SerializableHandle;
     use reqwest::blocking::Client;
-    use std::collections::HashMap;
     use url::Url;
 
+    use monolith::cache::Cache;
+    use monolith::core::Options;
     use monolith::html;
-    use monolith::opts::Options;
     use monolith::url::EMPTY_IMAGE_DATA_URL;
 
     #[test]
     fn basic() {
-        let cache = &mut HashMap::new();
+        let cache = &mut Some(Cache::new(0, None));
 
         let html: &str = "<div><P></P></div>";
         let dom = html::html_to_dom(&html.as_bytes().to_vec(), "".to_string());
@@ -51,7 +51,7 @@ mod passing {
         let html = "<div><P></P><iframe src=\"\"></iframe></div>";
         let dom = html::html_to_dom(&html.as_bytes().to_vec(), "".to_string());
         let url: Url = Url::parse("http://localhost").unwrap();
-        let cache = &mut HashMap::new();
+        let cache = &mut Some(Cache::new(0, None));
 
         let mut options = Options::default();
         options.silent = true;
@@ -79,7 +79,7 @@ mod passing {
         let html = "<frameset><frame src=\"\"></frameset>";
         let dom = html::html_to_dom(&html.as_bytes().to_vec(), "".to_string());
         let url: Url = Url::parse("http://localhost").unwrap();
-        let cache = &mut HashMap::new();
+        let cache = &mut Some(Cache::new(0, None));
 
         let mut options = Options::default();
         options.silent = true;
@@ -112,7 +112,7 @@ mod passing {
         ";
         let dom = html::html_to_dom(&html.as_bytes().to_vec(), "".to_string());
         let url: Url = Url::parse("http://localhost").unwrap();
-        let cache = &mut HashMap::new();
+        let cache = &mut Some(Cache::new(0, None));
 
         let mut options = Options::default();
         options.no_css = true;
@@ -153,7 +153,7 @@ mod passing {
                     <div><img src=\"http://localhost/assets/mono_lisa.png\" /></div>";
         let dom = html::html_to_dom(&html.as_bytes().to_vec(), "".to_string());
         let url: Url = Url::parse("http://localhost").unwrap();
-        let cache = &mut HashMap::new();
+        let cache = &mut Some(Cache::new(0, None));
 
         let mut options = Options::default();
         options.no_images = true;
@@ -195,7 +195,7 @@ mod passing {
             "<body background=\"no/such/image.png\" background=\"no/such/image2.png\"></body>";
         let dom = html::html_to_dom(&html.as_bytes().to_vec(), "".to_string());
         let url: Url = Url::parse("http://localhost").unwrap();
-        let cache = &mut HashMap::new();
+        let cache = &mut Some(Cache::new(0, None));
 
         let mut options = Options::default();
         options.no_images = true;
@@ -224,7 +224,7 @@ mod passing {
         let html = "<frameset><frame src=\"http://trackbook.com\"></frameset>";
         let dom = html::html_to_dom(&html.as_bytes().to_vec(), "".to_string());
         let url: Url = Url::parse("http://localhost").unwrap();
-        let cache = &mut HashMap::new();
+        let cache = &mut Some(Cache::new(0, None));
 
         let mut options = Options::default();
         options.no_frames = true;
@@ -261,7 +261,7 @@ mod passing {
         let html = "<iframe src=\"http://trackbook.com\"></iframe>";
         let dom = html::html_to_dom(&html.as_bytes().to_vec(), "".to_string());
         let url: Url = Url::parse("http://localhost").unwrap();
-        let cache = &mut HashMap::new();
+        let cache = &mut Some(Cache::new(0, None));
 
         let mut options = Options::default();
         options.no_frames = true;
@@ -302,7 +302,7 @@ mod passing {
         ";
         let dom = html::html_to_dom(&html.as_bytes().to_vec(), "".to_string());
         let url: Url = Url::parse("http://localhost").unwrap();
-        let cache = &mut HashMap::new();
+        let cache = &mut Some(Cache::new(0, None));
 
         let mut options = Options::default();
         options.no_js = true;
@@ -342,7 +342,7 @@ mod passing {
                     <link integrity=\"sha384-12345\" rel=\"something\" href=\"https://some-site.com/some-file.ext\" />";
         let dom = html::html_to_dom(&html.as_bytes().to_vec(), "".to_string());
         let url: Url = Url::parse("http://localhost").unwrap();
-        let cache = &mut HashMap::new();
+        let cache = &mut Some(Cache::new(0, None));
 
         let mut options = Options::default();
         options.silent = true;
@@ -382,7 +382,7 @@ mod passing {
         ";
         let dom = html::html_to_dom(&html.as_bytes().to_vec(), "".to_string());
         let url: Url = Url::parse("http://localhost").unwrap();
-        let cache = &mut HashMap::new();
+        let cache = &mut Some(Cache::new(0, None));
 
         let mut options = Options::default();
         options.no_css = true;
@@ -425,7 +425,7 @@ mod passing {
         ";
         let dom = html::html_to_dom(&html.as_bytes().to_vec(), "".to_string());
         let url: Url = Url::parse("http://localhost").unwrap();
-        let cache = &mut HashMap::new();
+        let cache = &mut Some(Cache::new(0, None));
 
         let mut options = Options::default();
         options.no_css = true;
@@ -474,7 +474,7 @@ mod passing {
         ";
         let dom = html::html_to_dom(&html.as_bytes().to_vec(), "".to_string());
         let url: Url = Url::parse("http://localhost").unwrap();
-        let cache = &mut HashMap::new();
+        let cache = &mut Some(Cache::new(0, None));
 
         let mut options = Options::default();
         options.no_css = true;
@@ -521,7 +521,7 @@ mod passing {
         </html>";
         let dom = html::html_to_dom(&html.as_bytes().to_vec(), "".to_string());
         let url: Url = Url::parse("http://localhost").unwrap();
-        let cache = &mut HashMap::new();
+        let cache = &mut Some(Cache::new(0, None));
 
         let mut options = Options::default();
         options.no_images = true;
@@ -562,7 +562,7 @@ mod passing {
         let html = "<script id=\"data\" type=\"application/json\">{\"mono\":\"lith\"}</script>";
         let dom = html::html_to_dom(&html.as_bytes().to_vec(), "".to_string());
         let url: Url = Url::parse("http://localhost").unwrap();
-        let cache = &mut HashMap::new();
+        let cache = &mut Some(Cache::new(0, None));
 
         let mut options = Options::default();
         options.silent = true;
