@@ -1,9 +1,7 @@
 mod cli;
 mod cookies;
+mod core;
 mod css;
 mod html;
 mod js;
-// mod macros;
-mod opts;
 mod url;
-mod utils;
